//******************************************************************************
//Name          : FIRWindow.cs
//Description   : Implementation to experience the beauty and the pitfalls of Filtering
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using System.Globalization;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Display;


class FIRFilter
{
    const int samplerate = 128;                     // Abtastwerte im Abtastintervall
    const double PI      =   3.141592653589793;

    public double[] a = new double[samplerate];	    // Filter kernel
    public double[] AS = new double[samplerate];    // Amplitudenspektrum
    public double[] PS = new double[samplerate];    // Phasenspektrum

    public int filterlength;

    public int Samplerate
    {
        get { return samplerate; }
    }

    public void LPKernel(double fc)
    {
        double sum = 0;

        for (int k=0; k<filterlength; k++)
        {
            // sinc-function
            a[k] = Math.Sin(1.0E-10 + 2*PI*fc*(k-filterlength/2))
                        /
                        (1.0E-10 + 2*PI*fc*(k-filterlength/2));

            // Hamming
            a[k] *= 0.54 - 0.46*Math.Cos(2*PI*k/filterlength);

            sum += a[k];
        }

        // normalize the filter-kernel
        for (int k=0; k<filterlength; k++)
        {
            a[k] /= sum;
        }
    }

    // HP = 1 - LP
    public void HPKernel(double fc)
    {
        // start the design with a Lowpass
        LPKernel(fc);

        // invert Lowpass
        for (int k=0; k<filterlength; k++)
        {
            a[k] = -a[k];
        }

        // add Allpass
        a[filterlength/2] += 1.0;
    }

    // BP = LP + HP - AP
    public void BPKernel(double fc1, double fc2)
    {
        // local copies
        double[] hlp = new double[filterlength];
        double[] hhp = new double[filterlength];

        LPKernel(fc2);
        for (int k=0; k<filterlength; k++)
        {
            hlp[k] = a[k];
        }

        HPKernel(fc1);
        for (int k=0; k<filterlength; k++)
        {
            hhp[k] = a[k];
        }

        // add Lowpass und Highpass
        for (int k=0; k<filterlength; k++)
        {
            a[k] = hlp[k] + hhp[k];
        }

        // minus Allpass
        a[filterlength/2] -= 1.0;
    }

    // BS = 1 - BP
    public void BSKernel(double fc1, double fc2)
    {
        // start the design with a Bandpass
        BPKernel(fc1, fc2);

        // invert Bandpass
        for (int k=0; k<filterlength; k++)
        {
            a[k] = -a[k];
        }

        // add Allpass
        a[filterlength/2] += 1.0;
    }

    public void FrequencyResponse(double startFreq, double stopFreq)
    {
        double deltaf = (stopFreq-startFreq)/samplerate;
        double freq   = startFreq;
        double real = 0;
        double imag = 0;

        for (int i=0; i<samplerate; i++)
        {
            real = 0;
            imag = 0;

            for (int k=0; k<samplerate; k++)
            {
                real += a[k]*Math.Cos(2*PI*freq*k/samplerate);
                imag += a[k]*Math.Sin(2*PI*freq*k/samplerate);
            }

            AS[i] = Math.Sqrt(real*real + imag*imag);
            PS[i] = Math.Atan2(imag, real);

            freq += deltaf;
        }
    }
}

class MainApp
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select = 0;
        double f1  = 0;
        double f2  = 0;

        while (select <= 4)
        {
            FIRFilter myFilter = new FIRFilter();

            for (int i=0; i<40; i++) Console.WriteLine();
            Console.Write("FIR-Filter design using windowed-sinc\n");
            Console.Write("Select one type of filter:\n\n");
            Console.Write("Lowpass                  ->  1\n");
            Console.Write("Highpass                 ->  2\n");
            Console.Write("Bandpass                 ->  3\n");
            Console.Write("Bandstop                 ->  4\n");
            userinput = Console.ReadLine();
            select = Convert.ToInt32(userinput);

            for (int i=0; i<40; i++) Console.WriteLine();

            if (select == 1)
            {
                Console.Write("Lowpass design\n");
                Console.Write("--------------\n");

                Console.Write("Please enter the filterlength (samplerate = {0}):\n", myFilter.Samplerate);
                Console.Write("filterlength = ");
                userinput = Console.ReadLine();
                myFilter.filterlength = Convert.ToInt32(userinput, CultureInfo.InvariantCulture);
                Console.WriteLine();

                Console.Write("Please enter the cut-off frequency related to samplerate:\n");
                Console.Write("fc/fs = ");
                userinput = Console.ReadLine();
                f1 = Convert.ToDouble(userinput, CultureInfo.InvariantCulture);
                myFilter.LPKernel(f1);
                Console.WriteLine();

                myFilter.FrequencyResponse(0, myFilter.Samplerate/2);
            }
            if (select == 2)
            {
                Console.Write("Highpass design\n");
                Console.Write("---------------\n");

                Console.Write("Please enter the filterlength (samplerate = {0}):\n", myFilter.Samplerate);
                Console.Write("filterlength = ");
                userinput = Console.ReadLine();
                myFilter.filterlength = Convert.ToInt32(userinput, CultureInfo.InvariantCulture);
                Console.WriteLine();

                Console.Write("Please enter the cut-off frequency related to samplerate:\n");
                Console.Write("fc/fs = ");
                userinput = Console.ReadLine();
                f1 = Convert.ToDouble(userinput, CultureInfo.InvariantCulture);
                myFilter.HPKernel(f1);
                Console.WriteLine();

                myFilter.FrequencyResponse(0, myFilter.Samplerate/2);
            }
            if (select == 3)
            {
                Console.Write("Bandpass design\n");
                Console.Write("---------------\n");

                Console.Write("Please enter the filterlength (samplerate = {0}):\n", myFilter.Samplerate);
                Console.Write("filterlength = ");
                userinput = Console.ReadLine();
                myFilter.filterlength = Convert.ToInt32(userinput);
                Console.WriteLine();

                Console.Write("Please enter the lower cut-off frequency related to samplerate:\n");
                Console.Write("fc1/fs = ");
                userinput = Console.ReadLine();
                f1 = Convert.ToDouble(userinput, CultureInfo.InvariantCulture);
                Console.Write("Please enter the upper cut-off frequency related to samplerate:\n");
                Console.Write("fc2/fs = ");
                userinput = Console.ReadLine();
                f2 = Convert.ToDouble(userinput, CultureInfo.InvariantCulture);
                myFilter.BPKernel(f1, f2);
                Console.WriteLine();

                myFilter.FrequencyResponse(0, myFilter.Samplerate/2);
            }
            if (select == 4)
            {
                Console.Write("Bandstop design\n");
                Console.Write("---------------\n");

                Console.Write("Please enter the filterlength (samplerate = {0}):\n", myFilter.Samplerate);
                Console.Write("filterlength = ");
                userinput = Console.ReadLine();
                myFilter.filterlength = Convert.ToInt32(userinput);
                Console.WriteLine();

                Console.Write("Please enter the lower cut-off frequency related to samplerate:\n");
                Console.Write("fc1/fs = ");
                userinput = Console.ReadLine();
                f1 = Convert.ToDouble(userinput, CultureInfo.InvariantCulture);
                Console.Write("Please enter the upper cut-off frequency related to samplerate:\n");
                Console.Write("fc2/fs = ");
                userinput = Console.ReadLine();
                f2 = Convert.ToDouble(userinput, CultureInfo.InvariantCulture);
                myFilter.BSKernel(f1, f2);
                Console.WriteLine();

                myFilter.FrequencyResponse(0, myFilter.Samplerate/2);
            }

            // create a chart for displaying that data...
            double[] iArg = new double[myFilter.Samplerate];  // integer argument to display filter kernel
            double[] dArg = new double[myFilter.Samplerate];  // double argument to display frequency response
            for (int k = 0; k < myFilter.Samplerate; k++)
            {
                iArg[k] = k;
                dArg[k] = (double)k/(double)(2);//*myFilter.Samplerate);
            }
            DisplayChart mydisplay1 = new DisplayChart(720, 480, SeriesChartType.Line, "Filter kernel", "k", "h(k)");
            mydisplay1.AddData(iArg, myFilter.a);

            DisplayChart mydisplay2 = new DisplayChart(720, 480, SeriesChartType.Line, "Frequency response", "f/fs", "|H(f)|");
            mydisplay2.AddData(dArg, myFilter.AS);

            // ...and run the chart-application
            Application.EnableVisualStyles();
            Application.Run(mydisplay1);
            Application.Run(mydisplay2);

            mydisplay1.Dispose();
            mydisplay2.Dispose();
            Application.Exit();
        }
    }
}
